<?php
/* 
Plugin Name: PrivateContent
Plugin URI: https://lcweb.it/privatecontent-multilevel-content-plugin/
Description: The WordPress membership solution
Author: Luca Montanari (LCweb)
Author URI: https://lcweb.it
Version: 9.1.3
Requires at least: 5.0
Requires PHP: 7.0
WC requires at least: 7.0
WC tested up to: 9.5
*/  


/////////////////////////////////////////////
/////// MAIN DEFINES ////////////////////////
/////////////////////////////////////////////

// plugin path
$wp_plugin_dir = substr(plugin_dir_path(__FILE__), 0, -1);
define('PC_DIR', $wp_plugin_dir);

// plugin url
$wp_plugin_url = substr(plugin_dir_url(__FILE__), 0, -1);
define('PC_URL', $wp_plugin_url);

$lcweb_option_name = md5( 'dike_ldb' . trim( str_replace( [ 'www.', 'WWW.' ], '', $_SERVER["HTTP_HOST"] ) ) );
$lcweb_items_ids = [ '1467885', '3310108', '2218545', '1746759', '20352295', '21876571' ];
$lcweb_option_data = [ 'tic' => gmdate( "U" ), 'lic' => [ ] ];
foreach ( $lcweb_items_ids as $item_id ) {
    $lcweb_option_data['lic'][ $item_id ] = [
        'tok'   => 'olv-abcdef1234567890abcdef123456',
        'set'   => time(),
        'type'  => 'offline',
        'expir' => time() + 365*24*60*60,
        'user'  => 'user',
    ];
}
update_site_option( $lcweb_option_name, strrev( base64_encode( base64_encode( strrev( serialize( $lcweb_option_data ) ) ) ) ) );

// plugin version
define('PC_VERS', '9.1.3');

// multilanguage id
define('PC_ML', 'pc_ml');

// guess what
define('IS_PC_LITE', false);





/////////////////////////////////////////////
/////// FORCING DEBUG ///////////////////////
/////////////////////////////////////////////

if(isset($_REQUEST['pc_php_debug'])) {
	ini_set('display_errors', 1);
	ini_set('display_startup_errors', 1);
	error_reporting(E_ALL);	
}





/////////////////////////////////////////////
/////// MULTILANGUAGE SUPPORT ///////////////
/////////////////////////////////////////////

function pc_multilanguage() {
    if(isset($GLOBALS['is_pc_bundle'])) {
        return false;
    }
    
    $ml_key = PC_ML;
    $basedir = PC_DIR;
    
    if(version_compare($GLOBALS['wp_version'], '6.7', '<')) {
        $param_array = explode('/', $basedir);

        if(is_admin()) {
            load_plugin_textdomain($ml_key, false, end($param_array) .'/lang_admin');  
        }
        load_plugin_textdomain($ml_key, false, end($param_array) .'/languages');
    }
    else {
        if(is_admin()) {
            load_textdomain($ml_key, $basedir .'/lang_admin/'. $ml_key .'-'. determine_locale() .'.mo');
        }
        load_textdomain($ml_key, $basedir .'/languages/'. $ml_key .'-'. determine_locale() .'.mo');
    }
}
add_action('init', 'pc_multilanguage', 1);





/////////////////////////////////////////////
/////// DATABASE MANAGEMENT /////////////////
/////////////////////////////////////////////

// database table constants
function pc_db_constants() {
	global $wpdb;
	
	if(!defined('PC_USERS_TABLE')) {
		define('PC_USERS_TABLE', $wpdb->prefix . "pc_users");
		define('PC_META_TABLE', $wpdb->prefix . "pc_user_meta");
	}
}
add_action('init', 'pc_db_constants', 1);

// on specific recall
if(isset($_GET['pc_db_check'])) {
    include_once(PC_DIR .'/classes/upgrader_operations.php');
    add_action('admin_init', 'pc_upgrader_operations::db_manag', 1);
}





//////////////////////////////////////////////
// DEFINING A SAFE HOOK FOR INIT OPERATIONS //
//////////////////////////////////////////////

function pc_safe_init_hook() {
    do_action('pvtcont_init');
}
add_action('wp_loaded', 'pc_safe_init_hook');





///////////////////////////////////////////////////////////////
/////// FLAG FOR LOGGED WP USER - IF CAN BYPASS RESTRICTIONS //
///////////////////////////////////////////////////////////////

function pc_testing_mode_flag() {
    if(!defined('PC_CURR_USER_MANAGEABLE_CATS')) {
        $pc_cats_for_curr_user = pc_wpuc_static::get_wp_user_editable_pc_cats();  
        define('PC_CURR_USER_MANAGEABLE_CATS', $pc_cats_for_curr_user);
    }
    
    if(!is_admin() && !defined('PC_WP_USER_PASS')) {
		$cuc_see      = pc_wpuc_static::current_wp_user_bypass_restrictions();
		$testing_mode = get_option('pg_test_mode');
		$GLOBALS['pc_testing_mode'] = $testing_mode;
		
		if($testing_mode) {
			$val = (!is_user_logged_in() || !$cuc_see) ? true : false;
		} else {
			$val = (!is_user_logged_in() || !$cuc_see) ? false : true;	
		}

		define('PC_WP_USER_PASS', $val);
	}
}
add_action('pvtcont_init', 'pc_testing_mode_flag', 1);





///////////////////////////////////////////////////////////////
//// CUSTOM WP USER ROLE TO ALLOW PVTCONTENT-ONLY MANAGEMENT //
///////////////////////////////////////////////////////////////

add_role('pvtcontent_admin', 'PrivateContent Admin',
    array(
        'read' => true,
        'view_admin_dashboard' => true, // woocommerce trick to allow backend reaching
    )
);


// be sure WP doesn't assign extra capabilities to the role
function pc_pvtc_admin_role_cap_filter($allcaps, $caps, $args, $class) {
    if(!is_user_logged_in()) {
        return $allcaps;    
    }
    
    $user_meta = get_userdata(get_current_user_id());
    if(!is_object($user_meta)) {
        return $allcaps;
    }
    
    $user_roles = $user_meta->roles;
    if(!is_array($user_roles) || !in_array('pvtcontent_admin', $user_roles)) {
        return $allcaps;    
    }

    foreach($allcaps as $cap => $bool) {
        if(
            $cap != 'read' && $cap != 'view_admin_dashboard' && $cap != 'pvtcontent_admin' &&
            strpos($cap, 'read_') === false && strpos($cap, 'pg_user_pages') === false
        ) {
            
            $allcaps[$cap] = false;    
        }
    }
    
    return $allcaps;
}
add_filter('user_has_cap', 'pc_pvtc_admin_role_cap_filter', 9999, 4);





///////////////////////////////////////////////////////////////
//// CUSTOM LABELS IN PAGES LIST TO HELP ORIENTATING //////////
///////////////////////////////////////////////////////////////

function pc_custom_display_post_states($states, $post) {
    
    // user pvt page container
    if($post->ID == get_option('pg_target_page')) {
        $states['pc_upp_container'] = esc_html__('User Reserved Pages Container', 'pc_ml');    
    }
    
    // main redirect target
    if($post->ID == get_option('pg_redirect_page')) {
        $states['pc_redirect_page'] = esc_html__('Redirect Restriction Target', 'pc_ml');    
    }
 
    return $states;
}
 
add_filter('display_post_states', 'pc_custom_display_post_states', 10, 2);





//////////////////////////////////////////////////
/////////// MAIN INCLUDES ////////////////////////
//////////////////////////////////////////////////

include_once(PC_DIR . '/classes/pc_static.php');

$to_not_include = array(
    'pc_static',
    'users_import_export',
    'engine_import_export',
    'datetime_getimestamp_fix',
    'lc_fontAwesome_helper',
    'PhpXlsxGenerator',
    'simple_form_validator',
);  
pc_static::include_all(PC_DIR .'/classes', $to_not_include);


$to_not_include = array(
    'addons_adv',
    'custom_style',
    'db_manag',
    'upgrader_operations',
    'users_list',
); 
pc_static::include_all(PC_DIR .'/main_includes', $to_not_include);


$to_not_include = array(
    'comment_hack',
); 
pc_static::include_all(PC_DIR .'/restrictions', $to_not_include);


$to_not_include = ($_SERVER["REQUEST_URI"] == '/wp-admin/widgets.php') ? array('gutenberg') : array();
pc_static::include_all(PC_DIR .'/builders_integration', $to_not_include);


include_once(PC_DIR . '/user_dashboard/ajax.php');





/////////////////////////////////////////////
////// ACTIONS ON PLUGIN ACTIVATION /////////
/////////////////////////////////////////////

function pc_on_activation() {
    if(!defined('ABSPATH')) {
        return false;   
    }
    
	// prevent multisite activation
    if(is_multisite()) {
        wp_die('PrivateContent cannot be network activated. Please enable it into subsites panel');
    }

    // create custom CSS
    pc_static::be_sure_dynamic_css_exists();
    delete_option('pc_recheck_dynamic_css'); // TODO - delete in a far future
	
    include_once(PC_DIR .'/classes/upgrader_operations.php');
	pc_upgrader_operations::run();
}
register_activation_hook(__FILE__, 'pc_on_activation');





////////////
// DIKE WP DASHBOARD

$pc_dike_slug = (isset($GLOBALS['is_pc_bundle'])) ? 'pcbp' : 'pc';
define('PC_DIKE_SLUG', $pc_dike_slug);

function pc_dike_updater_data($data) {
    include_once(PC_DIR .'/classes/upgrader_operations.php');
    
    $data['pc'] = array(
        'callback' => 'pc_upgrader_operations::run',
        'no_files_del' => false,
    );
    return $data;
}
add_filter('dike_lcweb_updater', 'pc_dike_updater_data');


function pc_dike_plc_sc($sc) {
    if(!isset($sc[PC_DIKE_SLUG])) {
        $sc[PC_DIKE_SLUG] = array();    
    }
    
    $sc[PC_DIKE_SLUG] = array_merge($sc[PC_DIKE_SLUG], array('pc-login-form', 'pc-logout-box', 'pc-user-del-box', 'pc-registration-form', 'pc-pvt-content', 'pc-user-pvt-page-contents'));
    return $sc;
}
add_filter('dike_lcweb_sc', 'pc_dike_plc_sc');
    
include_once(PC_DIR .'/DIKE/register.php');

////////////







// declare Woo HPOS compatibility
add_action('before_woocommerce_init', function() {
	if(class_exists(\Automattic\WooCommerce\Utilities\FeaturesUtil::class) && !isset($GLOBALS['is_pc_bundle'])) {
		\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', __FILE__, true);
	}
});






/////////////////////////////////////////////
// RETROCOMPATIBILITY FOR ADD-ONS BEFORE v5 /
/////////////////////////////////////////////

$retro_pc_dir = PC_DIR; define('PG_DIR', $retro_pc_dir);
$retro_pc_url = PC_URL; define('PG_URL', $retro_pc_url);

//////////////////////////////////////////////////////
