<?php
if(!defined('ABSPATH')) {exit;}


$user_statuses = array(
    'all'   => esc_html__('Any status', 'pc_ml'),
    '1'     => esc_html__('Active users', 'pc_ml'),
    '2'     => esc_html__('Disabled users', 'pc_ml'),
    '3'     => esc_html__('Pending users', 'pc_ml'),
);



$user_cats = array();
if(PC_CURR_USER_MANAGEABLE_CATS == 'any') {
    $user_cats["all"] = esc_html__('All categories', 'pc_ml');
}
foreach(get_terms('pg_user_categories', 'orderby=name&hide_empty=0') as $ucat) {
    if(!get_option('pg_tu_can_edit_user_cats') && PC_CURR_USER_MANAGEABLE_CATS != 'any' && !in_array($ucat->term_id, (array)PC_CURR_USER_MANAGEABLE_CATS)) {
        continue;    
    }
    $user_cats[ (int)$ucat->term_id ] = esc_html($ucat->name);
}






// validation indexes
$indexes = array();
$indexes[] = array('index'=>'pc_nonce', 'label'=>'nonce', 'required'=>true);
$indexes[] = array('index'=>'pc_csv_exp_export_type', 'label'=>esc_html__('Export type', 'pc_ml'), 'allowed'=>array('excel', 'csv'), 'required'=>true);

if(isset($_POST['pc_targeted_export_params'])) {
    $indexes[] = array('index'=>'pc_targeted_export_params', 'label'=>esc_html__('Advanced users search - export parameters', 'pc_ml'), 'required'=>true);
}
else {
    $indexes[] = array('index'=>'pc_csv_exp_user_statuses', 'label'=>esc_html__('Users type', 'pc_ml'), 'allowed'=>array_keys($user_statuses), 'required'=>true);
    $indexes[] = array('index'=>'pc_csv_exp_user_cats', 'label'=>esc_html__('User categories', 'pc_ml'), 'allowed'=>array_keys($user_cats), 'required'=>true);
}

// PC-FILTER - allow CSV import form validation array management. Array elements must comply with Simple form validator class
$indexes = apply_filters('pc_csv_export_form_validation', $indexes);


if(isset($_POST['pc_csv_export'])) {
    $validator = new simple_fv('pc_ml');
    $validator->formHandle($indexes);
    
    $fdata = $validator->form_val;
    
    if(!isset($fdata['pc_targeted_export_params'])) {
        if(!is_array($fdata['pc_csv_exp_user_statuses']) || in_array('all', $fdata['pc_csv_exp_user_statuses'])) {
            $fdata['pc_csv_exp_user_statuses'] = array('all');   
        }    
        if(!is_array($fdata['pc_csv_exp_user_cats']) || in_array('all', $fdata['pc_csv_exp_user_cats'])) {
            $fdata['pc_csv_exp_user_cats'] = array('all');   
        }
    }
    
    $error = $validator->getErrors();
    if(empty($error) && (!wp_verify_nonce($_POST['pc_nonce'], 'lcwp_nonce') || !pc_wpuc_static::current_wp_user_can_edit_pc_user('some'))) {
        $error = 'Cheating?';
    }
    if(!empty($error)) {
        echo '<div class="notice notice-error"><p>'. $error .'</p></div>';
    }
    
    
    // perform export
    else {
        $uie = new pc_users_import_export;
        $export_data = $uie->csv_export($fdata);
        
        if($export_data['status'] == 'error') {
            echo '<div class="notice notice-error"><p>'. $export_data['message'] .'</p></div>';
        }
        else {
            $filename = 'pvtcontent_on_'. strtolower(sanitize_title($_SERVER['HTTP_HOST'])) .' -'. wp_date('Y-m-d-H:i:s') .'.';
            $filename .= ($fdata['pc_csv_exp_export_type'] == 'csv') ? 'csv' : 'xlsx';
            
            $mime = ($fdata['pc_csv_exp_export_type'] == 'csv') ? 
                'text/csv; charset=utf-8' : 
                'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet; charset=utf-8';
            ?>

            <script type="text/javascript">
            (function() { 
                "use strict";
    
                let byteString = atob(`<?php echo str_replace('`', '\`', $export_data['filedata']) ?>`),
                    ab = new ArrayBuffer(byteString.length),
                    ia = new Uint8Array(ab);

                for(var i = 0; i < byteString.length; i++) {
                    ia[i] = byteString.charCodeAt(i);
                }
                
                const blob = new Blob([ab], {type: `<?php echo $mime ?>`}),
                      downloadUrl = URL.createObjectURL(blob),
                      a = document.createElement("a");
                
                a.href = downloadUrl;
                a.download = `<?php echo $filename ?>`;
                document.body.appendChild(a);
                a.click();
            })();
            </script>
            <?php
        }
    }
    
}
else {
    $fdata = array();
    foreach($indexes as $i) {
        $fdata[ $i['index'] ] = null; 
    }
    
    $fdata['pc_csv_exp_user_statuses'] = array('all');
    $fdata['pc_csv_exp_user_cats'] = array('all');
}
?>

<div class="pc_imp_exp_notices"></div>

<div class="pc_imp_exp_box">
    <h4><?php esc_html_e('Export configuration', 'pc_ml') ?></h4>
    
    <form method="post" class="form-wrap" action="<?php echo esc_attr(pc_static::curr_url()) ?>">
        <table class="widefat pc_imp_exp_table">
            <tbody>
                
                <?php if(!isset($_GET['pc_targeted_export']) || !isset($_GET['pc_texp_label']) || !isset($_GET['pc_texp_conds'])) : ?>
                    <tr>
                        <td><?php esc_html_e("Users status", 'pc_ml'); ?></td>
                        <td>
                            <select name="pc_csv_exp_user_statuses[]" class="pc_lc_select" data-placeholder="<?php esc_attr_e('Select an option', 'pc_ml') ?> .." multiple="multiple" autocomplete="off">
                                <?php
                                foreach($user_statuses as $key => $label) {
                                    $sel = (is_array($fdata['pc_csv_exp_user_statuses']) && in_array($key, $fdata['pc_csv_exp_user_statuses'])) ? 'selected="selected"' : '';
                                    echo '<option value="'. $key .'" '. $sel .'>'. $label .'</option>'; 
                                }
                                ?>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <td><?php esc_html_e("User categories", 'pc_ml'); ?></td>
                        <td>
                            <select name="pc_csv_exp_user_cats[]" class="pc_lc_select" data-placeholder="<?php esc_attr_e('Select categories', 'pc_ml') ?> .." multiple="multiple" autocomplete="off">
                                <?php
                                foreach($user_cats as $key => $label) {
                                    $sel = (is_array($fdata['pc_csv_exp_user_cats']) && in_array($key, $fdata['pc_csv_exp_user_cats'])) ? 'selected="selected"' : '';
                                    echo '<option value="'. $key .'" '. $sel .'>'. $label .'</option>'; 
                                }
                                ?>
                            </select>
                        </td>
                    </tr>
                <?php else : ?>
                    <tr>
                        <td><?php echo stripslashes(pc_static::sanitize_val($_GET['pc_texp_label'])) ?></td>
                        <td>
                            <?php echo stripslashes(pc_static::sanitize_val($_GET['pc_texp_conds'])) ?>
                            <input type="hidden" name="pc_targeted_export_params" value="<?php echo esc_attr($_GET['pc_targeted_export']) ?>" />
                        </td>
                    </tr>
                <?php endif; ?>
                
                <tr>
                    <td><?php esc_html_e("Export as", 'pc_ml') ?></td>
                    <td>
                        <select name="pc_csv_exp_export_type" class="pc_lc_select" data-placeholder="<?php esc_attr_e('Select an option', 'pc_ml') ?> .." autocomplete="off">
                            <option value="excel"><?php esc_html_e('Excel', 'pc_ml') ?> (.xls)</option>
                            <option value="csv" <?php selected('csv', $fdata['pc_csv_exp_export_type']) ?>>CSV</option>
                        </select>
                    </td>
                </tr>

                <?php
                // PC-ACTION - add fields in CSV export form - must comply with table code
                do_action('pc_csv_export_form', $fdata);
                ?>
                
            </tbody>
        </table>

        <br/>
        <input type="hidden" name="pc_nonce" value="<?php echo wp_create_nonce('lcwp_nonce') ?>" /> 
        <input type="submit" name="pc_csv_export" value="<?php esc_attr_e('Export', 'pc_ml') ?>" class="button-primary" />  
    </form>
</div>


<script type="text/javascript">
(function($) { 
    "use strict"; 
    
    $(document).on('change', 'select[name="pc_csv_exp_user_cats[]"], select[name="pc_csv_exp_user_statuses[]"]', function() {
        let vals = ($(this).val() !== null && typeof($(this).val()) == "object") ? $(this).val() : [];
        
        // if ALL is selected, discard the rest
        if(vals.includes('all')) {
            $(this).find("option").prop("selected", false);
            $(this).find('option[value="all"]').prop("selected", true);

            const resyncEvent = new Event("lc-select-refresh");
            this.dispatchEvent(resyncEvent);
        }

        const resyncEvent = new Event('lc-select-refresh');
        this.dispatchEvent(resyncEvent);
    });
})(jQuery);
</script>